/*:
@target MV MZ
@plugindesc スキルツリーコンフィグ v1.0.1
@author うなぎおおとろ
@url https://raw.githubusercontent.com/unagiootoro/RPGMZ/master/SkillTreeConfig.js

@help
[概要]
スキルツリーを導入するプラグインです。
SPを使ってスキルを習得するスキルツリーを作成することができます。

[使用方法]
■ スキルツリーの設定
スキルツリーの設定は、「SkillTreeConfig.js」ファイルを編集することで行います。
基本的な設定としては、アクターごとにスキルツリーのタイプ(剣スキルや魔法スキルなど)を設定し、
そしてタイプごとにスキルツリーを構築します。
スキルツリーの構築は、スキルの派生設定(ファイアⅠを取得したらファイアⅡが取得可能になるなど)によって行います。

■ SPの入手設定
スキルの習得には、SPが必要となります。
SPの入手方法としては
・戦闘終了による獲得
・レベルアップによるSP獲得
の二通りの設定を行うことができます。

・戦闘終了時に得られるSPの設定方法
敵キャラのメモ欄に
<battleEndGainSp: SP>
の形式で記載します。

・レベルアップによるSP獲得方法の設定
コンフィグの「levelUpGainSp」によって設定を行います。

■ イベントでSPを獲得する方法
スクリプトで
skt_gainSp(アクターID, 獲得するSP値)
と記載することで、該当のアクターが指定したSPを獲得することができます。
例えば、アクターIDが1のアクターが5SPを獲得する場合、
skt_gainSp(1, 5);
と記載します。

■ 割り振った累計SPの取得
skt_totalSp(アクターID, 累計SP格納先変数ID)
と記載することで、該当のアクターが今までに割り振ったSPを指定した変数に代入することができます。
例えば、アクターIDが1のアクターの累計SPをID2の変数に代入する場合、
skt_totalSp(1, 2);
と記載します。

■ スキルリセット
スクリプトで
skt_skillReset(アクターID);
と記載することで、一度習得したスキルをリセットすることができます。
例えば、アクターIDが1のアクターのスキルリセットを行う場合、
skt_skillReset(1);
と記載します。

■ スキルツリータイプの有効/無効
スクリプトで
skt_enableType(アクターID, "タイプ名");
と記載することで、タイプを有効にします。

無効にする場合は、
skt_disableType(アクターID, "タイプ名")
と記載します。

無効にしたタイプは、スキルツリーのタイプ一覧には表示されません。

■ タイプの引継ぎ
特定の条件を満たすとスキルツリーに新たなスキルが追加されるようにしたい場合、「タイプの引継ぎ」を使用します。
例えば、タイプ「下位魔法」を「上位魔法」に変更したい場合、あらかじめ両方のタイプをコンフィグに登録した上で、
「上位魔法」は無効化しておきます。そして、タイプの引継ぎ機能を用いて、「下位魔法」を「上位魔法」に引き継がせます。

タイプの引継ぎを行う場合、スクリプトで
skt_migrationType(アクターID, "引継ぎ元タイプ名", "引継ぎ先タイプ名", リセット有無);
と記載します。リセット有無については、引継ぎ後、引継ぎ元のタイプのスキルツリーをリセットする場合、trueを、
リセットしない場合、falseを指定します。
例えば、アクターIDが1のアクターが、タイプ「下位魔法」を「上位魔法」に引き継がせ、さらにスキルリセットを行う場合、
skt_migrationType(アクターID, "下位魔法", "上位魔法", true);
と記載します。

■ マップからスキルツリーを読み込む
マップからスキルツリーの各スキルの配置座標を読み込むことで、ある程度自由なレイアウトのスキルツリーを
作成することができます。この機能によって設定可能なのはスキルの座標のみであり、スキル間の線はプラグイン側で描画します。

・スキル座標の設定
マップ上のイベントにて、設定を行います。
例えば、"ファイア"というスキルがある場合、スキルを配置したい座標に空のイベントを作成し、
イベントのメモ欄に
ファイア
と記載します。すると、"ファイア"とメモ欄に記載したイベントのXY座標がスキルのXY座標として使用されます。

■ スクリプトからスキルツリーを起動
スクリプトで
skt_open(アクターID);
と記載することで、指定したアクターのスキルツリーを起動することができます。

■ スクリプトからスキルツリーのスキルを取得する
スクリプトで
skt_learn(アクターID, "タイプ名", スキル名, 強制取得有無(省略可));
と記載することで、指定したスキルを取得することができます。
スキル名は「skillTreeInfo」で指定したものを使用してください。
強制取得有無にtrueを設定した場合、習得可能か否かの判定を無視して強制的にスキルを取得します。
また、この場合SPは消費されません。
強制取得有無にfalseを設定した場合は通常通りの方法でスキルを取得します。この場合はSPも消費されます。
なおこの項目については省略可能です。省略した場合はfalseが適用されます。
例: アクターID1のキャラクターで、タイプ「剣技」のスキルツリーの「強撃」を取得する場合
skt_learn(1, "剣技", "強撃", false);
*/

const loadSkillTreeConfig = () => {
return {
// =============================================================
// ●ここからは設定項目です。
// =============================================================

// スキルツリーのタイプの設定を行います。
// skillTreeTypes: [ ～ ]の中にアクターの数だけタイプ設定を追加します。

// タイプ設定は、次の形式で設定します。
// { actorId: アクターのID, types: [タイプ情報1, タイプ情報2, ...] }

// タイプ情報は、次の形式で設定します。
// [タイプ種別, タイプ名, タイプの説明, タイプ有効/無効]
// タイプ種別...スキルの派生設定でタイプを識別するためのユニークな識別子を設定します。
// タイプ名...タイプ一覧のウィンドウに表示するタイプ名を設定します。
// タイプの説明...タイプ一覧のウィンドウに表示するタイプの説明を設定します。
// タイプ有効/無効...タイプを有効にする場合は、trueを、無効にする場合は、falseを指定します。
//                  この項目については、省略可能です。省略した場合、trueが指定されます。
skillTreeTypes: [
    {
        actorId: 10,
        types: [
            ["剣技", "剣技", "剣技を取得します。", true, 76],
            ["攻撃魔法", "攻撃魔法", "攻撃魔法を取得します。", true, 79],
            ["回復魔法", "回復魔法", "回復魔法を取得します", true, 80],
            ["パッシブスキル（オリヴィア）", "パッシブスキル", "パッシブスキルを取得します", true, 81],            
        ]
    },

    {
        actorId: 11,
        types: [
            ["短剣術", "短剣術", "短剣術を取得します", true, 82],
            ["攻撃魔法", "攻撃魔法", "攻撃魔法を取得します。", true, 85],
            ["回復魔法", "回復魔法", "回復魔法を取得します", true, 86],
            ["パッシブスキル（ミリア）", "パッシブスキル", "パッシブスキルを取得します", true, 87],            
        ]
    },

    
],

// スキルツリーのマップ読み込み設定を行います。
// 読み込むマップは以下の形式で指定します。
// skillTreeMapId: { skillTreeName1: mapID1, skillTreeName2: mapID2, ... }
// skillTreeName...スキルツリーのタイプ名を指定します。
// mapID...読み込むマップIDを指定します。0の場合は読み込みを行いません。
skillTreeMapId: {
    "剣技": 2,
    "短剣術": 3,
    "攻撃魔法":4,
    "回復魔法":5,
    "パッシブスキル（オリヴィア）":6,
    "パッシブスキル（ミリア）":56
},

// 各スキルの情報を登録します。
// skillTreeInfo: [ ～ ]の中に登録するスキル数分のスキル情報の登録を行います。

// スキル情報の登録は次の形式で行います。
// [スキル名, スキルID, 必要SP, アイコン情報]
// スキル名...スキルツリーの派生設定でスキルを一意に識別するための識別子
//            識別子なので、実際のスキル名と一致していなくても問題はありません。
// スキルID...データベース上で該当するスキルのID
// 必要SP...スキルの習得に必要なSP
// アイコン情報については、アイコンを使用するか、任意の画像を使用するかに応じて次の形式で登録します。
//   アイコンを使用する場合 ["icon", iconIndex]
//   iconIndex...使用するアイコンのインデックス
//               iconIndexは省略可能です。省略した場合、スキルに設定されているアイコンが使用されます。
//   画像を使用する場合 ["img", fileName]
//   fileName...画像のファイル名。画像は、「img/pictures」フォルダにインポートしてください。
// なお、アイコン情報については省略可能です。省略した場合、["icon"]が適用されます。
skillTreeInfo: [
    // 剣技
    ["強撃", 172, 10, ["icon"]],
    ["薙ぎ払い", 173, 10, ["icon"]],
    ["連続攻撃", 174, 10, ["icon"]],
    ["強撃＋", 175, 30, ["icon"]],
    ["薙ぎ払い＋", 176, 30, ["icon"]],
    ["連続攻撃＋", 177, 30, ["icon"]],
    ["剣神の一撃", 178, 50, ["icon"]],

    //短剣術
    ["刺突", 233, 10, ["icon"]],
    ["刺突＋", 234, 30, ["icon"]],
    ["剣の舞", 235, 10, ["icon"]],
    ["剣の舞＋", 236, 30, ["icon"]],
    ["3段突き", 237, 10, ["icon"]],
    ["3段突き＋", 238, 30, ["icon"]],
    ["極み・3連突", 239, 50, ["icon"]],

    //攻撃魔法
    ["マジックアローⅠ", 92, 10, ["icon"]],
    ["マジックアローⅡ", 93, 20, ["icon"]],

    ["アローレインⅠ", 95, 10, ["icon"]],
    ["アローレインⅡ", 96, 20, ["icon"]],

    ["バーストⅠ", 153, 30, ["icon"]],
    ["バーストⅡ", 154, 50, ["icon"]],
    ["ニュークリアⅠ", 156, 30, ["icon"]],
    ["ニュークリアⅡ", 157, 50, ["icon"]],

    // 回復魔法
    ["ヒールⅠ", 52, 10, ["icon"]],
    ["ヒールⅡ", 53, 20, ["icon"]],
    ["ヒールⅢ", 54, 30, ["icon"]],

    ["リカバーⅠ", 56, 10, ["icon"]],
    ["リカバーⅡ", 57, 20, ["icon"]],
    ["リカバーⅢ", 58, 30, ["icon"]],

    ["レイズⅠ", 64, 20, ["icon"]],
    ["レイズⅡ", 65, 30, ["icon"]],

    //パッシブスキル
    ["攻撃UP(小)", 243, 10, ["icon"]],
    ["攻撃UP(中)", 244, 30, ["icon"]],
    ["攻撃UP(大)", 245, 50, ["icon"]],

    ["防御UP(小)", 246, 10, ["icon"]],
    ["防御UP(中)", 247, 30, ["icon"]],
    ["防御UP(大)", 248, 50, ["icon"]],

    ["魔攻UP(小)", 249, 10, ["icon"]],
    ["魔攻UP(中)", 250, 30, ["icon"]],
    ["魔攻UP(大)", 251, 50, ["icon"]],

    ["魔防UP(小)", 252, 10, ["icon"]],
    ["魔防UP(中)", 253, 30, ["icon"]],
    ["魔防UP(大)", 254, 50, ["icon"]],

    ["敏捷UP(小)", 255, 10, ["icon"]],
    ["敏捷UP(中)", 256, 30, ["icon"]],
    ["敏捷UP(大)", 257, 50, ["icon"]],

    ["取得経験値UP(小)", 258, 10, ["icon"]],
    ["取得経験値UP(中)", 259, 30, ["icon"]],
    ["取得経験値UP(大)", 260, 50, ["icon"]],

    ["ステータスUP(小)", 264, 10, ["icon"]],
    ["ステータスUP(中)", 265, 30, ["icon"]],
    ["ステータスUP(大)", 266, 70, ["icon"]],

    ["HP UP(小)", 289, 10, ["icon"]],
    ["HP UP(中)", 290, 30, ["icon"]],
    ["HP UP(大)", 291, 50, ["icon"]],

    ["MP UP(小)", 292, 10, ["icon"]],
    ["MP UP(中)", 293, 30, ["icon"]],
    ["MP UP(大)", 294, 50, ["icon"]],
],

// スキルツリーの派生設定を行います。
// skillTreeDerivative: { ～ }の中にタイプ数分のスキルツリーの登録を行います。

// スキルツリーの派生設定は次のように行います。
// "タイプ名": [ [スキル1, [派生先スキル1, 派生先スキル2, ...]], [スキル2, [派生先スキル3, 派生先スキル4, ...] ]
// ※派生先スキルが存在しない終端スキルの場合、派生先スキルは省略可能です。
//
// 例えば、"様子を見る"と"連続攻撃"を取得すると、"２回攻撃"が取得できるようにするには、次の設定を行います。
// ["様子を見る", ["２回攻撃"]],
// ["連続攻撃", ["２回攻撃"]],
// ["２回攻撃"],
//
// また、"ヒール"を取得すると、"ファイア"と"スパーク"が取得できるようにするには、次の設定を行います。
// ["ヒール", ["ファイア"]],
// ["ヒール", ["スパーク"]],
// ["ファイア"],
// ["スパーク"],
skillTreeDerivative: {
    "剣技": [
        ["強撃", ["強撃＋"]],
        ["薙ぎ払い", ["薙ぎ払い＋"]],
        ["連続攻撃", ["連続攻撃＋"]],
        ["強撃＋", ["剣神の一撃"]],
        ["薙ぎ払い＋", ["剣神の一撃"]],
        ["連続攻撃＋", ["剣神の一撃"]],
        ["剣神の一撃"],
    ],

    "短剣術": [
        ["刺突", ["刺突＋"]],
        ["剣の舞", ["剣の舞＋"]],
        ["3段突き", ["3段突き＋"]],
        ["刺突＋", ["極み・3連突"]],
        ["剣の舞＋", ["極み・3連突"]],
        ["3段突き＋", ["極み・3連突"]],
        ["極み・3連突"],
    ],

    "攻撃魔法": [
        ["マジックアローⅠ", ["マジックアローⅡ", "アローレインⅠ"]],
        ["マジックアローⅡ", ["バーストⅠ"]],
        ["アローレインⅠ", ["アローレインⅡ"]],
        ["アローレインⅡ", ["バーストⅠ"]],
        ["バーストⅠ", ["バーストⅡ", "ニュークリアⅠ"]],
        ["バーストⅡ"],
        ["ニュークリアⅠ", ["ニュークリアⅡ"]],
        ["ニュークリアⅡ"],
     ],


    "回復魔法": [
        ["ヒールⅠ", ["ヒールⅡ", "リカバーⅠ"]],
        ["ヒールⅡ", ["ヒールⅢ"]],
        ["リカバーⅠ", ["リカバーⅡ"]],
        ["ヒールⅢ", ["リカバーⅢ"]],
        ["リカバーⅡ", ["リカバーⅢ"]],

        ["レイズⅠ", ["レイズⅡ"]],
        ["リカバーⅢ", ["レイズⅡ"]],
        ["レイズⅡ"],
    ],

    "パッシブスキル（オリヴィア）": [
        ["HP UP(小)", ["HP UP(中)"]],
        ["HP UP(中)", ["HP UP(大)"]],
        ["HP UP(大)"],

        ["MP UP(小)", ["MP UP(中)"]],
        ["MP UP(中)", ["MP UP(大)"]],
        ["MP UP(大)"],

        ["攻撃UP(小)", ["攻撃UP(中)"]],
        ["攻撃UP(中)", ["攻撃UP(大)"]],
        ["攻撃UP(大)"],

        ["防御UP(小)", ["防御UP(中)"]],
        ["防御UP(中)", ["防御UP(大)"]],
        ["防御UP(大)"],

        ["魔攻UP(小)", ["魔攻UP(中)"]],
        ["魔攻UP(中)", ["魔攻UP(大)"]],
        ["魔攻UP(大)"],

        ["魔防UP(小)", ["魔防UP(中)"]],
        ["魔防UP(中)", ["魔防UP(大)"]],
        ["魔防UP(大)"],

        ["敏捷UP(小)", ["敏捷UP(中)"]],
        ["敏捷UP(中)", ["敏捷UP(大)"]],
        ["敏捷UP(大)"],

        ["取得経験値UP(小)", ["取得経験値UP(中)"]],
        ["取得経験値UP(中)", ["取得経験値UP(大)"]],
        ["取得経験値UP(大)"],
    ],

    "パッシブスキル（ミリア）": [
        ["HP UP(小)", ["HP UP(中)"]],
        ["HP UP(中)", ["HP UP(大)"]],
        ["HP UP(大)"],

        ["MP UP(小)", ["MP UP(中)"]],
        ["MP UP(中)", ["MP UP(大)"]],
        ["MP UP(大)"],

        ["攻撃UP(小)", ["攻撃UP(中)"]],
        ["攻撃UP(中)", ["攻撃UP(大)"]],
        ["攻撃UP(大)"],

        ["防御UP(小)", ["防御UP(中)"]],
        ["防御UP(中)", ["防御UP(大)"]],
        ["防御UP(大)"],

        ["魔攻UP(小)", ["魔攻UP(中)"]],
        ["魔攻UP(中)", ["魔攻UP(大)"]],
        ["魔攻UP(大)"],

        ["魔防UP(小)", ["魔防UP(中)"]],
        ["魔防UP(中)", ["魔防UP(大)"]],
        ["魔防UP(大)"],

        ["敏捷UP(小)", ["敏捷UP(中)"]],
        ["敏捷UP(中)", ["敏捷UP(大)"]],
        ["敏捷UP(大)"],

        ["ステータスUP(小)", ["ステータスUP(中)"]],
        ["ステータスUP(中)", ["ステータスUP(大)"]],
        ["ステータスUP(大)"],
    ]

},

// レベルアップによってSPを獲得する場合、レベルごとに得られるSP値を以下の形式で設定します。
// classId: 職業ID, default: デフォルト値, レベル: SP値, レベル: SP値, ...
// 下記の設定例では、レベル2では3SP取得、レベル3では4SP取得、それ以外のレベルでは5SPを獲得します。
levelUpGainSp: [
    {
        classId: 1,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 2,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 3,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 4,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 5,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 6,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 7,
        default: 5,
        2: 3,
        3: 4,
    },

    {
        classId: 8,
        default: 5,
        2: 3,
        3: 4,
    },
]
// =============================================================
// ●設定項目はここまでです。
// =============================================================
};
};
